@@ -1,5 +1,5 @@
 #/*
-# *  Copyright 2007 hkrn <hikarin@users.sourceforge.jp>
+# *  Copyright 2007-2009 hkrn <hikarin@users.sourceforge.jp>
 # *
 # *  Licensed under the Apache License, Version 2.0 (the "License");
 # *  you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: BBS.pm 1483 2007-10-25 12:59:40Z hikarin $
+# $Id: BBS.pm 1861 2009-06-28 14:47:41Z hikarin $
 #
 
 package Img0ch::App::BBS;
@@ -161,7 +161,7 @@ sub get_host {
 
     if (   $iRequest->get_header('via') =~ $re
         || $iRequest->get_header('x-forwarded-for') =~ $re
-        || $iRequest->get_header('forwarded')       =~ $re )
+        || $iRequest->get_header('forwarded') =~ $re )
     {
         my $proxy = gethostbyaddr( pack( 'C4', ( $1, $2, $3, $4 ) ), 2 )
             || join( '.', $1, $2, $3, $4 );
@@ -177,8 +177,12 @@ sub get_host {
     if ( $iRequest->is_mobile_device($mode) ) {
         my $serial = $iApp->{_serial} = $iRequest->get_device_id($mode);
         if ($serial) {
-            if ( !$iApp->config()->get('DisableBBMCheck')
-                and $iApp->is_bbm( $serial, $iRequest->now() ) )
+            if (!$iApp->config()->get('DisableBBMCheck')
+                and $iApp->is_bbm(
+                    $iApp->_imode_id_for_dns( $serial, $mode ),
+                    $iRequest->now()
+                )
+                )
             {
                 $iApp->{__error} = 'DENY_FROM_BBM_LISTED_HOST';
                 return 0;
@@ -200,9 +204,30 @@ sub validate {
     my $iKernel  = $iApp->{__kernel};
     my $iSetting = $iApp->{__setting};
     my $iRequest = $iApp->{_request};
-    my $charset  = $iKernel->get_encoding(1);
+    my $charset  = $iBBS->get_encoding();
     my $utf8 = $iRequest->param('utf8') ? 'utf8' : undef;
-    my ( $name, $mail, $comment, $subject );
+    my ( $comment, $subject );
+
+    my $name = $iRequest->param( 'FROM', 1 );
+    $name =~ tr/\r\n\t//d;
+    $iApp->escape_chars( \$name, 1 );
+    $iApp->trip(
+        \$name,
+        $iSetting->get_int('BBS_TRIPCOLUMN'),
+        $iKernel->get_encoded_str(
+            $iKernel->translate_symbol('TrueTrip'),
+            $charset, 'utf8'
+        )
+    );
+
+    my $mail = $iRequest->param('mail');
+    $mail =~ tr/\r\n\t//d;
+    $mail =~ s/&(?:amp;)?\#(\d+;)/&&$1/gxms;
+    $mail =~ s/\#(.+)\z//xms;
+    my $pass = $1 || '';
+    my $iCap = $iBBS->get_cap_instance($pass);
+    $iCap->load();
+    $iCap->is_valid();
 
     if ( $charset eq 'sjis' ) {
 
@@ -213,10 +238,8 @@ sub validate {
                 [\x81-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]//gxms;
             return length $str ? 0 : 1;
         };
-        $name = $iRequest->param('FROM');
         ( $utf8 or !$is_sjis_function->($name) )
             and $name = $iKernel->get_encoded_str( $name, $charset, $utf8 );
-        $mail = $iRequest->param('mail');
         ( $utf8 or !$is_sjis_function->($mail) )
             and $mail = $iKernel->get_encoded_str( $mail, $charset, $utf8 );
         $comment = $iRequest->param('MESSAGE');
@@ -229,23 +252,14 @@ sub validate {
             = $iKernel->get_encoded_str( $subject, $charset, $utf8 );
     }
     else {
-        $name = $iKernel->get_encoded_str( $iRequest->param('FROM'),
-            $charset, $utf8 );
-        $mail = $iKernel->get_encoded_str( $iRequest->param('mail'),
-            $charset, $utf8 );
+        $name = $iKernel->get_encoded_str( $name, $charset, $utf8 );
+        $mail = $iKernel->get_encoded_str( $mail, $charset, $utf8 );
         $comment = $iKernel->get_encoded_str( $iRequest->param('MESSAGE'),
             $charset, $utf8 );
         $subject = $iKernel->get_encoded_str( $iRequest->param('subject'),
             $charset, $utf8 );
     }
 
-    $mail =~ s/&(?:amp;)?\#(\d+;)/&&$1/gxms;
-    $mail =~ s/\#(.+)\z//xms;
-    my $pass = $1 || '';
-    my $iCap = $iBBS->get_cap_instance($pass);
-    $iCap->load();
-    $iCap->is_valid();
-
     my $enable_unicode = $iSetting->get('BBS_UNICODE') eq 'change' ? 0 : 1;
     $enable_unicode
         ? $mail =~ s/&&(\d+;)/&#$1/gxms
@@ -259,8 +273,6 @@ sub validate {
         }
     }
 
-    $name =~ tr/\r\n\t//d;
-    $mail =~ tr/\r\n\t//d;
     my $mkthread = $iApp->{__mkthread};
     if ( !$mkthread and index( $mail, 'sage' ) >= 0 ) {
         $iApp->{__sage}            = 1;
@@ -288,22 +300,19 @@ sub validate {
         $name = $iSetting->get('BBS_NONAME_NAME');
         $iApp->{__param_FROM} = $name;
     }
-    $iApp->escape_chars( \$name, 1 );
-    $iApp->trip(
-        \$name,
-        $iSetting->get_int('BBS_TRIPCOLUMN'),
-        $iConfig->get('multibyte.TRUE_TRIP')
-    );
 
     my $host = $iApp->{_host};
     $name =~ s|\Afusianasan|</b>$host<b>|xms;
     $name =~ s|fusianasan| </b>$host<b>|gxms;
 
     if ($capn) {
-        my $atmark = $iConfig->get('multibyte.ATMARK')
-            || pack( 'C*', ( 0x81, 0x97 ) );
-        my $star = $iConfig->get('multibyte.TRUE_CAP')
-            || pack( 'C*', ( 0x81, 0x9a ) );
+        my $atmark
+            = $iKernel->get_encoded_str( $iKernel->translate_symbol('Atmark'),
+            $charset, 'utf8' );
+        my $star
+            = $iKernel->get_encoded_str(
+            $iKernel->translate_symbol('TrueCap'),
+            $charset, 'utf8' );
         if ($name) {
             $name .= $atmark . $capn . ' ' . $star;
         }
@@ -364,8 +373,8 @@ sub validate {
         $comment = join "\n", @new_comment;
     }
 
-    $comment =~ s/\A\s+//xms;
-    $comment =~ s/\s+\z//xms;
+    $comment =~ s/\A[ \f\t]+//xms;
+    $comment =~ s/[ \f\t]+\z//xms;
     $comment =~ s/\r\n/\n/gxms;
     $comment =~ s/\n/<br>/gxms;
     $comment =~ tr/\r\t//d;
@@ -416,7 +425,9 @@ sub is_writeable {
         }
     }
 
-    my $resmax = $iApp->{__resmax} = $iConfig->get_int('ResMAX')
+    my $resmax 
+        = $iApp->{__resmax} 
+        = $iConfig->get_int('ResMAX')
         || $iSetting->get_int('BBS_RES_MAX')
         || 1000;
     if ( $iApp->{__mkthread} ) {
@@ -506,8 +517,7 @@ sub get_date {
         $iApp->{__bbs}->get_name() );
 
     if ( $iSetting->is_checked('BBS_SLIP') ) {
-        $mode = [ '0', 'O', 'O', 'O', 'O', 'Q', 'Q' ]->[ $iApp->{__agent} ];
-        $host =~ /p2\.2ch\.net/xms and $mode = 'P';
+        $mode = $iApp->{_request}->get_device_mark( $iApp->{__agent}, $host );
     }
     if ( $iSetting->is_checked('BBS_NO_ID') ) {
         $id = length $mode ? " ${mode}" : '';
@@ -525,18 +535,17 @@ sub get_date {
         $id = $iApp->{__sage} ? " ID:???${mode}" : " ID:${id}${mode}";
     }
 
+    my $iKernel = $iApp->{__kernel};
+    my $charset = $iApp->{__bbs} ? $iApp->{__bbs}->get_encoding() : undef;
     $iApp->{__param_id}   = $id;
     $iApp->{__param_date} = $iApp->get_date_string(
         $now, $$,
         ( $iConfig->get('DateFormat') || '%Y/%m/%d(%w) %H:%M:%S' ),
-        (   $iSetting->get('BBS_YMD_WEEKS') || pack(
-                'C*',
-                (   0x93, 0xfa, 0x2f, 0x8c, 0x8e, 0x2f, 0x89, 0xce,
-                    0x2f, 0x90, 0x85, 0x2f, 0x96, 0xd8, 0x2f, 0x8b,
-                    0xe0, 0x2f, 0x93, 0x79,
-                )
+        (   $iSetting->get('BBS_YMD_WEEKS') || $iKernel->get_encoded_str(
+                $iKernel->translate_symbol('Week'),
+                $charset, 'utf8'
             )
-        ),
+        )
     );
 
     return 1;
@@ -554,22 +563,36 @@ sub post {
         'DisableBBSPresavePlugin' )
         or return 0;
 
-    my $key      = $iApp->{__key};
-    my $iRequest = $iApp->{_request};
-    my $iSetting = $iApp->{__setting};
-    my $mkthread = $iApp->{__mkthread};
-    my $mode     = $iSetting->get('BBS_MODE');
-    if ((      ( $mode eq 'picture' and !$mkthread )
-            or ( $mode eq 'news'    and $mkthread )
+    my $key        = $iApp->{__key};
+    my $iRequest   = $iApp->{_request};
+    my $iSetting   = $iApp->{__setting};
+    my $mkthread   = $iApp->{__mkthread};
+    my $mode       = $iSetting->get('BBS_MODE');
+    my $uploadable = (
+               ( $mode eq 'picture' and !$mkthread )
+            or ( $mode eq 'news' and $mkthread )
             or
             ( $iSetting->is_checked('BBS_IMG_UPLOAD_THREAD') and $mkthread )
-        )
-        and $iRequest->is_uploadable('file')
-        )
-    {
+    );
+
+    if ( $uploadable and $iRequest->is_uploadable('file') ) {
         $iApp->_upload( $count, $key, $iBBS, $iConfig, $iRequest, $iSetting )
             or return 0;
     }
+    elsif ( $uploadable
+        and $mkthread
+        and $iSetting->is_checked('BBS_IMG_REQUIRE_UPLOADING_THREAD') )
+    {
+        $iApp->{__error} = 'UPLOAD_WITHOUT_FILE';
+        return 0;
+    }
+    elsif ( $uploadable
+        and !$mkthread
+        and $iSetting->is_checked('BBS_IMG_REQUIRE_UPLOADING_RES') )
+    {
+        $iApp->{__error} = 'UPLOAD_WITHOUT_FILE';
+        return 0;
+    }
 
     my $iLog = $iBBS->get_log_instance($key);
     $iLog->add(
@@ -810,6 +833,17 @@ sub get_poster_id {
         0, 8;
 }
 
+sub get_day_id {
+    my ( $iApp, $now, @argument ) = @_;
+    my @time = localtime($now);
+    push @argument, ( $time[5], $time[4], $time[3] + 31 );
+    if ( $iApp->{_host} =~ /(?:p2\.2ch\.net|razil\.jp)\z/xms ) {
+        my $ua = $iApp->{_request}->get_header('user-agent');
+        push @argument;
+    }
+    return substr Digest::MD5::md5_base64(@argument), 0, 8;
+}
+
 *id = \&get_poster_id;
 
 sub get_date_string {
@@ -844,67 +878,116 @@ sub trip {
     my ( $iClass, $name, $column, $trip ) = @_;
     $trip ||= pack( 'C*', ( 0x81, 0x9f ) );
 
-    $$name =~ s/&(?:amp;)?\#(\d+;)/&&$1/gxms;
     if ( $$name =~ /\A(.*?)\#(.+)\z/xms ) {
-        my ( $nm, $key ) = ( $1, $2 );
-        my $salt = substr( substr( $key, 0, 8 ) . 'H.', 1, 2 );
-        $salt =~ s/[^\.-z]/\./gxms;
-        $salt =~ tr/\x3A-\x40\x5B-\x60\x00-\x2D\x7B-\xFF/A-Ga-f./;
-        $column = -1 * $column;
-        my $t = substr crypt( $key, $salt ), $column;
+        my ( $nm, $key ) = ( Img0ch::Kernel::escape_html_entities($1), $2 );
+        my $t;
+        if ( length $key >= 12 ) {
+            my $mark = ord($key);
+            if ( $mark == ord('#') || $mark == ord('$') ) {
+                if ( $key
+                    =~ m{\A\#([[:xdigit:]]{16})([\./0-9A-Za-z]{0,2})\z}xms )
+                {
+                    $t = substr crypt( pack( 'H*', $1 ), $2 . '..' ),
+                        -1 * $column;
+                }
+                else {
+                    $t = '???';
+                }
+            }
+            else {
+                if ( defined $Digest::SHA1::VERSION
+                    or eval 'require Digest::SHA1' )
+                {
+                    $t = substr Digest::SHA1::sha1_base64($key), 0, 12;
+                }
+                else {
+                    defined $Digest::SHA::PurePerl::VERSION
+                        or require Digest::SHA::PurePerl;
+                    $t = substr Digest::SHA::PurePerl::sha1_base64($key), 0,
+                        12;
+                }
+                $t =~ tr/+/./;
+            }
+        }
+        else {
+            my $salt = substr( substr( $key, 0, 8 ) . 'H.', 1, 2 );
+            $salt =~ s/[^\.-z]/\./gxms;
+            $salt =~ tr/\x3A-\x40\x5B-\x60\x00-\x2D\x7B-\xFF/A-Ga-f./;
+            $t = substr crypt( $key, $salt ), -1 * $column;
+        }
+        $nm = Img0ch::Kernel::escape_html_entities($nm);
         $$name = $nm ? "${nm} </b>${trip}${t} <b>" : "</b>${trip}${t} <b>";
-        $$name =~ s/&&(\d+;)/&#$1/gxms;
         return 1;
     }
+    $$name = Img0ch::Kernel::escape_html_entities($$name);
 
-    $$name =~ s/&&(\d+;)/&#$1/gxms;
     return 0;
 }
 
 sub escape_chars {
-    my ( $iApp, $data, $f, $iConfig ) = @_;
-    my @q = ( 0x81, 0x68 );
+    my ( $iApp, $data, $f ) = @_;
+    my $iKernel = $iApp->{__kernel};
+    my $charset = $iApp->{__bbs} ? $iApp->{__bbs}->get_encoding() : undef;
 
-    $iConfig ||= $iApp->{_config};
-    my $cap1 = $iConfig->get('multibyte.TRUE_CAP')
-        || pack( 'C*', ( 0x81, 0x9a ) );
-    my $cap0 = $iConfig->get('multibyte.FALSE_CAP')
-        || pack( 'C*', ( 0x81, 0x99 ) );
-    my $tr1 = $iConfig->get('multibyte.TRUE_TRIP')
-        || pack( 'C*', ( 0x81, 0x9f ) );
-    my $tr0 = $iConfig->get('multibyte.FALSE_TRIP')
-        || pack( 'C*', ( 0x81, 0x9e ) );
-    my $rcv1 = $iConfig->get('multibyte.RECOVER')
-        || pack( 'C*', ( 0x95, 0x9c, 0x8b, 0x41 ) );
-    my $rcv0 = $iConfig->get('multibyte.RECOVER_ESCAPED')
-        || pack( 'C*', ( @q, 0x95, 0x9c, 0x8b, 0x41, @q ) );
-    my $del1 = $iConfig->get('multibyte.DELETER')
-        || pack( 'C*', ( 0x8d, 0xed, 0x8f, 0x9c ) );
-    my $del0 = $iConfig->get('multibyte.DELETER_ESCAPED')
-        || pack( 'C*', ( @q, 0x8d, 0xed, 0x8f, 0x9c, @q ) );
-    my $ldr1 = $iConfig->get('multibyte.LEADER')
-        || pack( 'C*', ( 0x88, 0xc4, 0x93, 0xe0 ) );
-    my $ldr0 = $iConfig->get('multibyte.LEADER_ESCAPED')
-        || pack( 'C*', ( @q, 0x88, 0xc4, 0x93, 0xe0, @q ) );
-    $$data =~ s/$cap1/$cap0/gxms;
-    $$data =~ s/$tr1/$tr0/gxms;
-    $$data =~ s/$rcv1/$rcv0/gxms;
-    $$data =~ s/$del1/$del0/gxms;
-    $$data =~ s/$ldr1/$ldr0/gxms;
+    my $cap1
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('TrueCap'),
+        $charset, 'utf8' );
+    my $cap0
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('FalseCap'),
+        $charset, 'utf8' );
+    my $tr1
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('TrueTrip'),
+        $charset, 'utf8' );
+    my $tr0
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('FalseTrip'),
+        $charset, 'utf8' );
+    my $rcv1
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('Recover'),
+        $charset, 'utf8' );
+    my $rcv0
+        = $iKernel->get_encoded_str(
+        $iKernel->translate_symbol('RecoverEscaped'),
+        $charset, 'utf8' );
+    my $del1
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('Deleter'),
+        $charset, 'utf8' );
+    my $del0
+        = $iKernel->get_encoded_str(
+        $iKernel->translate_symbol('DeleterEscaped'),
+        $charset, 'utf8' );
+    my $ldr1
+        = $iKernel->get_encoded_str( $iKernel->translate_symbol('Leader'),
+        $charset, 'utf8' );
+    my $ldr0
+        = $iKernel->get_encoded_str(
+        $iKernel->translate_symbol('LeaderEscaped'),
+        $charset, 'utf8' );
+    $$data =~ s/\Q$cap1\E/$cap0/gxms;
+    $$data =~ s/\Q$tr1\E/$tr0/gxms;
+    $$data =~ s/\Q$rcv1\E/$rcv0/gxms;
+    $$data =~ s/\Q$del1\E/$del0/gxms;
+    $$data =~ s/\Q$ldr1\E/$ldr0/gxms;
 
     if ($f) {
-        my $adm1 = $iConfig->get('multibyte.ADMIN')
-            || pack( 'C*', ( 0x8a, 0xc7, 0x97, 0x9d ) );
-        my $adm0 = $iConfig->get('multibyte.ADMIN_ESCAPED')
-            || pack( 'C*', ( @q, 0x8a, 0xc7, 0x97, 0x9d, @q ) );
-        my $adn1 = $iConfig->get('multibyte.ADNIN')
-            || pack( 'C*', ( 0x8a, 0xc7, 0x92, 0xbc ) );
-        my $adn0 = $iConfig->get('multibyte.ADNIN_ESCAPED')
-            || pack( 'C*', ( @q, 0x8a, 0xc7, 0x92, 0xbc, @q ) );
-        $$data =~ s/$adm1/$adm0/gxms;
-        $$data =~ s/$adn1/$adn0/gxms;
+        my $adm1
+            = $iKernel->get_encoded_str( $iKernel->translate_symbol('Admin'),
+            $charset, 'utf8' );
+        my $adm0
+            = $iKernel->get_encoded_str(
+            $iKernel->translate_symbol('AdminEscaped'),
+            $charset, 'utf8' );
+        my $adn1
+            = $iKernel->get_encoded_str( $iKernel->translate_symbol('Adnin'),
+            $charset, 'utf8' );
+        my $adn0
+            = $iKernel->get_encoded_str(
+            $iKernel->translate_symbol('AdninEscaped'),
+            $charset, 'utf8' );
+        $$data =~ s/\Q$adm1\E/$adm0/gxms;
+        $$data =~ s/\Q$adn1\E/$adn0/gxms;
     }
-    1;
+
+    return 1;
 }
 
 sub is_referer {
@@ -1124,7 +1207,7 @@ sub _upload {
     my ( $iApp, $resno, $key, $iBBS, $iConfig, $iRequest, $iSetting ) = @_;
     my $filename = $iRequest->filename('file');
     my $ext;
-    if ( !$iApp->{__cap}->can_post_without_limit()
+    if (   !$iApp->{__cap}->can_post_without_limit()
         and $iRequest->fsize() > $iSetting->get_int('BBS_IMG_MAX_SIZE') )
     {
         $iApp->{__error} = 'UPLOAD_FILESIZE_OVER';
@@ -1154,7 +1237,7 @@ sub _upload {
     }
 
     my $remove_key = $iRequest->param('pass');
-    if ( !$remove_key
+    if (   !$remove_key
         and $iSetting->is_checked('BBS_IMG_REQUIRE_REMOVE_KEY') )
     {
         $iApp->{__error} = 'UPLOAD_WITHOUT_REMOVE_KEY';
@@ -1247,8 +1330,7 @@ sub redirect_success {
                 'Referer'      => $iRequest->get_header('referer'),
             }
         );
-        my $charset = $iConfig->get('DefaultCharset');
-        $iRequest->send_http_header( 'text/html', $charset );
+        $iRequest->send_http_header( 'text/html', $iBBS->get_encoding() );
         $iTemplate->flush(
             $is_mobile
             ? sub { ${ $_[0] } =~ tr/\n//d; ${ $_[0] } }
@@ -1263,7 +1345,8 @@ sub redirect_error {
     my ( $iApp, $template, $cs, $filter, $callback ) = @_;
     my $iConfig  = $iApp->{_config};
     my $iKernel  = $iApp->{__kernel};
-    my $iObject  = $iApp->{__bbs} || $iKernel;
+    my $iBBS     = $iApp->{__bbs};
+    my $iObject  = $iBBS || $iKernel;
     my $iRequest = $iApp->{_request};
 
     defined &{'Img0ch::Error::new'} or require Img0ch::Error;
@@ -1277,14 +1360,11 @@ sub redirect_error {
         $long =~ s/{!$subs!}/$value/gxms;
     }
 
-    my $charset = $iKernel->get_encoding(1);
-    my $ip      = $iApp->{_host} || $iApp->{_ip} || $iRequest->ip();
-    my $name    = $iApp->{__param_FROM}
-        || $iKernel->get_encoded_str( $iRequest->param('FROM'), $charset );
-    my $mail = $iApp->{__param_mail}
-        || $iKernel->get_encoded_str( $iRequest->param('mail'), $charset );
+    my $charset = $iBBS ? $iBBS->get_encoding() : $iKernel->get_encoding(1);
+    my $ip = $iApp->{_host} || $iApp->{_ip} || $iRequest->ip();
     my $message = $iApp->{__param_MESSAGE}
-        || $iKernel->get_encoded_str( $iRequest->param('MESSAGE'), $charset );
+        || $iKernel->get_encoded_str( ( $iRequest->param('MESSAGE') || '' ),
+        $charset );
     my $data = {
         ok     => 0,
         error  => $error,
@@ -1302,11 +1382,24 @@ sub redirect_error {
     }
     else {
         my $iSetting = $iApp->{__setting};
-        $iApp->trip(
-            \$name,
-            ( $iSetting ? $iSetting->get_int('BBS_TRIPCOLUMN') : 10 ),
-            $iConfig->get('multibyte.TRUE_TRIP'),
-        );
+        my $name     = $iApp->{__param_FROM};
+        if ( !$name ) {
+            $name = $iRequest->param( 'FROM', 1 );
+            $iApp->escape_chars( \$name, 1 );
+            $iApp->trip(
+                \$name,
+                ( $iSetting ? $iSetting->get_int('BBS_TRIPCOLUMN') : 10 ),
+                $iKernel->get_encoded_str(
+                    $iKernel->translate_symbol('TrueTrip'), $charset,
+                    'utf8'
+                )
+            );
+            $iKernel->get_encoded_str( $name, $charset );
+        }
+        my $mail = $iApp->{__param_mail}
+            || $iKernel->get_encoded_str( ( $iRequest->param('mail') || '' ),
+            $charset );
+        $mail =~ s/\#.*//xms;
         my $iTemplate = Img0ch::Template->new(
             $iObject,
             {   file => ( $template || 'error' ),
@@ -1329,8 +1422,8 @@ sub redirect_error {
                 'ShortErrorStr' => $long,
             }
         );
-        my $charset = $cs || $iConfig->get('DefaultCharset');
-        $iRequest->send_http_header( 'text/html', $charset );
+        $cs = $iBBS ? $iBBS->get_encoding() : undef;
+        $iRequest->send_http_header( 'text/html', $cs );
         $iTemplate->flush($filter);
     }
 
@@ -1344,10 +1437,10 @@ sub _update_index_thread {
 
     {
         my $first = $iThread->get(1);
-        my $id    = $first->[3];
-        my $date  = $id ? ( join ' ', $first->[2], $id ) : $first->[2];
         my $text  = $first->[4];
-        $iter->( $key, $count, 1, \$text );
+        $iter->( $key, $count, 1, \$text, $first );
+        my $id = $first->[3];
+        my $date = $id ? ( join ' ', $first->[2], $id ) : $first->[2];
         push @{$reses},
             {
             'num'     => 1,
@@ -1370,10 +1463,10 @@ sub _update_index_thread {
 
     for ( my $i = $start; $i <= $iterate; $i++ ) {
         my $res  = $iThread->get($i);
-        my $id   = $res->[3];
-        my $date = $id ? ( join ' ', $res->[2], $id ) : $res->[2];
         my $text = $res->[4];
-        $iter->( $key, $count, $i, \$text );
+        $iter->( $key, $count, $i, \$text, $res );
+        my $id = $res->[3];
+        my $date = $id ? ( join ' ', $res->[2], $id ) : $res->[2];
         push @{$reses},
             {
             'num'     => $i,
@@ -1395,6 +1488,20 @@ sub _update_index_anchors {
     else                     { ( $_[0] - 1, $_[0] + 1 ) }
 }
 
+sub _imode_id_for_dns {
+    my ( $iApp, $serial, $mode ) = @_;
+    if ( $mode == 1 ) {
+        my $i = 0;
+        my @s = unpack( 'C*', $serial );
+        my $j = scalar @s;
+        $serial .= '-';
+        while ( $i < $j ) {
+            $serial .= ( ( $s[ $i++ ] & 96 ) == 96 ) ? 1 : 0;
+        }
+    }
+    return $serial;
+}
+
 1;
 __END__
 
