@@ -1,5 +1,5 @@
 #/*
-# *  Copyright 2007 hkrn <hikarin@users.sourceforge.jp>
+# *  Copyright 2007-2009 hkrn <hikarin@users.sourceforge.jp>
 # *
 # *  Licensed under the Apache License, Version 2.0 (the "License");
 # *  you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: XCSV.pm 717 2007-05-29 13:18:34Z hikarin $
+# $Id: XCSV.pm 1826 2009-04-08 15:23:25Z hikarin $
 #
 
 package Img0ch::Plugin::Upload::XCSV;
@@ -25,26 +25,22 @@ sub new {
     my ( $iClass, $iBBS, $key, $called ) = @_;
     $called eq 'iterator' or return;
 
-    require Img0ch::Thread;
-    require Img0ch::Upload;
-    my $iThread = Img0ch::Thread->new( $iBBS, $key );
-    my $iUpload = Img0ch::Upload->new( $iBBS, $key );
-
     bless {
         __kernel => $iBBS->get_kernel(),
-        __thread => $iThread,
-        __upload => $iUpload,
+        __thread => $iBBS->get_thread_instance($key),
+        __upload => $iBBS->get_upload_instance($key),
     }, $iClass;
 }
 
 sub generate {
     my ($iApp) = @_;
-    _process( $iApp->kernel(),
-        $iApp->upload(), $iApp->thread()->count() + 1 );
+    _process( $iApp->kernel(), $iApp->upload(),
+        $iApp->thread()->count() + 1 );
     return 1;
 }
 
 *batch = \&repair;
+
 sub repair {
     my ( $iObject, $key ) = @_;
     my $iKernel = $iObject->{__kernel};
