@@ -1,5 +1,5 @@
 #/*
-# *  Copyright 2007 hkrn <hikarin@users.sourceforge.jp>
+# *  Copyright 2007-2009 hkrn <hikarin@users.sourceforge.jp>
 # *
 # *  Licensed under the Apache License, Version 2.0 (the "License");
 # *  you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: User.pm 389 2007-03-20 14:00:43Z hikarin $
+# $Id: User.pm 1826 2009-04-08 15:23:25Z hikarin $
 #
 
 package Zeromin::User;
@@ -68,6 +68,9 @@ BEGIN {
         can_edit_res
         can_abone_res
         can_remove_res
+        can_freeze_file
+        can_release_file
+        can_remove_poll
         can_enter_archive_section
         can_create_archive
         can_remove_archive
@@ -92,6 +95,7 @@ BEGIN {
         can_view_error_log
         can_view_thread_log
         can_view_create_log
+        can_view_zeromin_log
         can_enter_plugin_section
         can_regist_plugin
         can_config_plugin
@@ -149,10 +153,11 @@ sub save {
     my $iRepos = $zUser->{__repos};
 
     my $ucount = $zUser->get_user_count();
-    my $up = $zUser->{__user_pool};
+    my $up     = $zUser->{__user_pool};
     for my $id ( keys %{$up} ) {
         my $value = $up->{$id};
         my $name = $value->{name} || '';
+        $name =~ tr/\r\n//d;
         $iRepos->set( "I:U:I.${id}.name", $name );
         $iRepos->set( "I:U:I.${id}.mail", $value->{mail} || '' );
         $iRepos->set( "I:U:I.${id}.pass", $value->{pass} || '' );
@@ -166,6 +171,7 @@ sub save {
         my $value     = $gp->{$id};
         my $name      = $value->{name} || '';
         my $privilege = $value->{privilege};
+        $name =~ tr/\r\n//d;
         $iRepos->set( "I:U:G.${id}.name",        $name );
         $iRepos->set( "I:U:G.${id}.role",        $value->{role} || 1 );
         $iRepos->set( "I:U:G.${id}.role.user",   $privilege->{user} || 0 );
@@ -207,21 +213,27 @@ sub is_valid {
 
     $pass or return 0;
     $name ||= $zUser->{__name};
-    my $iRepos = $zUser->{__repos};
-    my $uid    = Digest::MD5::md5_base64($name);
-    $iRepos->get("I:U:I.${uid}.pass") eq $pass or return 0;
+    my $iRepos  = $zUser->{__repos};
+    my $uid_b64 = Digest::MD5::md5_base64($name);
+    my $uid_hex = Digest::MD5::md5_hex($name);
+    $iRepos->get("I:U:I.${uid_hex}.pass") eq $pass
+        or $iRepos->get("I:U:I.${uid_b64}.pass") eq $pass
+        or return 0;
 
-    my $bbs    = $zUser->{__bbs};
-    my $gid    = $iRepos->get_int("I:U:I.${uid}.gid") or return 0;
+    my $bbs = $zUser->{__bbs};
+    my $gid = $iRepos->get_int("I:U:I.${uid_hex}.gid")
+        || $iRepos->get_int("I:U:I.${uid_b64}.gid")
+        or return 0;
     my $global = $iRepos->get("I:U:G:S.${gid}.0");
-    my $scope  = $iRepos->get("I:U:G:S.${gid}.${bbs}") || $global;
+    my $scope = $iRepos->get("I:U:G:S.${gid}.${bbs}") || $global;
     $global and $zUser->{__bbs} = 0;
 
     if ( !$scope ) {
-        $zUser->{__id}        = $uid;
-        $zUser->{__gid}       = $gid;
-        $zUser->{__name}      = $name;
-        $zUser->{__mail}      = $iRepos->get("I:U:I.${uid}.mail");
+        $zUser->{__id}   = $uid_hex;
+        $zUser->{__gid}  = $gid;
+        $zUser->{__name} = $name;
+        $zUser->{__mail} = $iRepos->get("I:U:I.${uid_hex}.mail")
+            || $iRepos->get("I:U:I.${uid_b64}.mail");
         $zUser->{__pass}      = $pass;
         $zUser->{__can_login} = $iRepos->get_int("I:U:G.${gid}.role");
         return 1;
@@ -276,10 +288,13 @@ sub is_valid {
         $zUser->{__can_recover_pool}       = $r_pool & 2 ? 1 : 0;
         $zUser->{__can_remove_pool}        = $r_pool & 4 ? 1 : 0;
         my $r_res = $iRepos->get_int("I:U:G.${gid}.role.res");
-        $zUser->{__can_enter_res_section} = $r_res     ? 1 : 0;
-        $zUser->{__can_abone_res}         = $r_res & 1 ? 1 : 0;
-        $zUser->{__can_remove_res}        = $r_res & 2 ? 1 : 0;
-        $zUser->{__can_edit_res}          = $r_res & 4 ? 1 : 0;
+        $zUser->{__can_enter_res_section} = $r_res      ? 1 : 0;
+        $zUser->{__can_abone_res}         = $r_res & 1  ? 1 : 0;
+        $zUser->{__can_remove_res}        = $r_res & 2  ? 1 : 0;
+        $zUser->{__can_edit_res}          = $r_res & 4  ? 1 : 0;
+        $zUser->{__can_freeze_file}       = $r_res & 8  ? 1 : 0;
+        $zUser->{__can_release_file}      = $r_res & 16 ? 1 : 0;
+        $zUser->{__can_remove_poll}       = $r_res & 32 ? 1 : 0;
         my $r_archive = $iRepos->get_int("I:U:G.${gid}.role.archive");
         $zUser->{__can_enter_archive_section} = $r_archive     ? 1 : 0;
         $zUser->{__can_create_archive}        = $r_archive & 1 ? 1 : 0;
@@ -310,14 +325,16 @@ sub is_valid {
         $zUser->{__can_view_error_log}    = $r_log & 1 ? 1 : 0;
         $zUser->{__can_view_create_log}   = $r_log & 2 ? 1 : 0;
         $zUser->{__can_view_thread_log}   = $r_log & 4 ? 1 : 0;
+        $zUser->{__can_view_zeromin_log}  = $r_log & 8 ? 1 : 0;
         my $r_plugin = $iRepos->get_int("I:U:G.${gid}.role.plugin");
         $zUser->{__can_enter_plugin_section} = $r_plugin     ? 1 : 0;
         $zUser->{__can_regist_plugin}        = $r_plugin & 1 ? 1 : 0;
         $zUser->{__can_config_plugin}        = $r_plugin & 2 ? 1 : 0;
-        $zUser->{__id}                       = $uid;
+        $zUser->{__id}                       = $uid_hex;
         $zUser->{__gid}                      = $gid;
         $zUser->{__name}                     = $name;
-        $zUser->{__mail} = $iRepos->get("I:U:G.${uid}.mail");
+        $zUser->{__mail} = $iRepos->get("I:U:I.${uid_hex}.mail")
+            || $iRepos->get("I:U:I.${uid_b64}.mail");
         $zUser->{__pass} = $pass;
         return 1;
     }
@@ -329,7 +346,7 @@ sub get_bbs_range {
     $user_id ||= $zUser->{__id};
 
     my $iRepos = $zUser->{__repos};
-    my $gid = $zUser->get_user_gid($user_id);
+    my $gid    = $zUser->get_user_gid($user_id);
     $iRepos->get("I:U:G:S.${gid}.0") and return [];
 
     my $result = [];
@@ -341,7 +358,8 @@ sub get_bbs_range {
             }
             return 0;
         },
-        $gid, $result,
+        $gid,
+        $result,
     );
     return $result;
 }
@@ -357,10 +375,10 @@ sub get_current_user {
 
 sub get_user_count {
     my ($zUser) = @_;
-    my $iRepos = $zUser->{__repos};
-    my $count  = $iRepos->get_int('I:U:I._');
+    my $iRepos  = $zUser->{__repos};
+    my $count   = $iRepos->get_int('I:U:I._');
 
-    if (!$count) {
+    if ( !$count ) {
         my $result = {};
         $iRepos->iterate(
             sub {
@@ -381,7 +399,7 @@ sub have_privilege {
     my ( $zUser, $bbs_id ) = @_;
     my $iRepos = $zUser->{__repos};
     my $gid    = $zUser->{__gid};
-    $bbs_id    = Img0ch::Kernel::intval($bbs_id);
+    $bbs_id = Img0ch::Kernel::intval($bbs_id);
     $iRepos->get("I:U:G:S.${gid}.${bbs_id}")
         || $iRepos->get("I:U:G:S.${gid}.0");
 }
@@ -390,7 +408,9 @@ sub add {
     my ( $zUser, $user ) = @_;
     $zUser->get_group_name( $user->{gid} ) or return 0;
     my $name = $user->{name};
-    my $uid  = Digest::MD5::md5_base64($name);
+    $name =~ tr/\r\n//d;
+    $user->{name} = $name;
+    my $uid = Digest::MD5::md5_hex($name);
     $zUser->get_user_name($uid) eq $name and return 0;
     $zUser->_plugin( 'zeromin.create.user', [ $uid, $user ] );
     $zUser->{__user_pool}->{$uid} = $user;
@@ -502,7 +522,7 @@ sub get_users {
 sub get_users_with_page {
     my ( $zUser, $item_per_page, $offset ) = @_;
 
-    require Data::Page;
+    defined $Data::Page::VERSION or require Data::Page;
     my $entries = $zUser->get_users();
     my $page = Data::Page->new( scalar @$entries, $item_per_page, $offset );
     return ( [ $page->splice($entries) ], $page );
@@ -563,6 +583,7 @@ sub add_group {
             $seen->{$scope} = 1;
         }
     }
+    $group->{name} =~ tr/\r\n//d;
     $group->{bbs} = [ grep { $_ != 0 } keys %$seen ];
     @{ $group->{bbs} } or @{ $group->{bbs} } = (0);
 
@@ -807,7 +828,7 @@ sub get_groups {
 sub get_groups_with_page {
     my ( $zUser, $item_per_page, $offset ) = @_;
 
-    require Data::Page;
+    defined $Data::Page::VERSION or require Data::Page;
     my $entries = $zUser->get_groups();
     my $page = Data::Page->new( scalar @$entries, $item_per_page, $offset );
     return ( [ $page->splice($entries) ], $page );
@@ -871,7 +892,26 @@ sub _get {
     else {
         my $prefix = { 'user' => 'I:U:I', 'group' => 'I:U:G' }->{$type};
         my $iRepos = $zUser->{__repos};
-        $iRepos->get("${prefix}.${id}.${accessor}");
+        my $ret    = $iRepos->get("${prefix}.${id}.${accessor}");
+        if ( !$ret ) {
+            my $new_id;
+            if ( defined $MIME::Base64::VERSION
+                or eval 'require MIME::Base64; 1' )
+            {
+                $new_id
+                    = MIME::Base64::encode_base64( pack( 'H*', $id ), '' );
+            }
+            else {
+                defined $MIME::Base64::Perl::VERSION
+                    or require MIME::Base64::Perl;
+                $new_id
+                    = MIME::Base64::Perl::encode_base64( pack( 'H*', $id ),
+                    '' );
+            }
+            $new_id =~ tr/=//d;
+            $ret = $iRepos->get("${prefix}.${new_id}.${accessor}");
+        }
+        return $ret;
     }
 }
 
